@@ -1,10 +1,10 @@
-from typing import IO, Dict, Generator, Generic, List, TypeVar, Union
+from typing import IO, Dict, Generator, Generic, List, Optional, TypeVar, Union
 
 from scaleapi.batches import Batch, BatchStatus
 from scaleapi.evaluation_tasks import EvaluationTask
 from scaleapi.exceptions import ScaleInvalidRequest
 from scaleapi.files import File
-from scaleapi.projects import Project
+from scaleapi.projects import Project, TaskTemplate
 from scaleapi.training_tasks import TrainingTask
 
 from ._version import __version__  # noqa: F401
@@ -48,9 +48,22 @@ class Batchlist(Paginator[Batch]):
 class ScaleClient:
     """Main class serves as an interface for Scale API"""
 
-    def __init__(self, api_key, source=None, api_instance_url=None):
+    def __init__(
+        self,
+        api_key,
+        source=None,
+        api_instance_url=None,
+        verify=None,
+        proxies=None,
+        cert=None,
+    ):
         self.api = Api(
-            api_key, user_agent_extension=source, api_instance_url=api_instance_url
+            api_key,
+            user_agent_extension=source,
+            api_instance_url=api_instance_url,
+            verify=verify,
+            proxies=proxies,
+            cert=cert,
         )
 
     def get_task(self, task_id: str) -> Task:
@@ -148,6 +161,49 @@ class ScaleClient:
         endpoint = f"task/{task_id}/setMetadata"
         return Task(self.api.post_request(endpoint, body=metadata), self)
 
+    def set_task_tags(self, task_id: str, tags: List[str]) -> Task:
+        """Sets completely new list of tags to a task and returns the
+        associated task.
+        Args:
+            task_id (str):
+                Task id
+            tags (List[str]):
+                List of new tags to set
+        Returns:
+            Task
+        """
+        endpoint = f"task/{task_id}/tags"
+        return Task(self.api.post_request(endpoint, body=tags), self)
+
+    def add_task_tags(self, task_id: str, tags: List[str]) -> Task:
+        """Adds a list of tags to a task and returns the
+        associated task.
+        Args:
+            task_id (str):
+                Task id
+            tags (List[str]):
+                List of tags to add.
+                Already present tags will be ignored.
+        Returns:
+            Task
+        """
+        endpoint = f"task/{task_id}/tags"
+        return Task(self.api.put_request(endpoint, body=tags), self)
+
+    def delete_task_tags(self, task_id: str, tags: List[str]) -> Task:
+        """Deletes a list of tags from a task and returns the
+        associated task.
+        Args:
+            task_id (str):
+                Task id
+            tags (List[str]):
+                List of tags to delete. Nonpresent tags will be ignored.
+        Returns:
+            Task
+        """
+        endpoint = f"task/{task_id}/tags"
+        return Task(self.api.delete_request(endpoint, body=tags), self)
+
     def tasks(self, **kwargs) -> Tasklist:
         """Returns a list of your tasks.
         Returns up to 100 at a time, to get more, use the
@@ -213,6 +269,14 @@ class ScaleClient:
             limit (int):
                 Determines the page size (1-100)
 
+            include_attachment_url (bool):
+                If true, returns a pre-signed s3 url for the
+                attachment used to create the task.
+
+            limited_response (bool):
+                If true, returns task response of the following fields:
+                task_id, status, metadata, project, otherVersion.
+
             next_token (str):
                 Can be use to fetch the next page of tasks
         """
@@ -232,6 +296,8 @@ class ScaleClient:
             "updated_before",
             "updated_after",
             "unique_id",
+            "include_attachment_url",
+            "limited_response",
         }
 
         for key in kwargs:
@@ -254,7 +320,7 @@ class ScaleClient:
 
     def get_tasks(
         self,
-        project_name: str,
+        project_name: str = None,
         batch_name: str = None,
         task_type: TaskType = None,
         status: TaskStatus = None,
@@ -267,6 +333,9 @@ class ScaleClient:
         created_after: str = None,
         created_before: str = None,
         tags: Union[List[str], str] = None,
+        include_attachment_url: bool = True,
+        limited_response: bool = None,
+        limit: int = None,
     ) -> Generator[Task, None, None]:
         """Retrieve all tasks as a `generator` method, with the
         given parameters. This methods handles pagination of
@@ -276,7 +345,7 @@ class ScaleClient:
         `task_list = list(get_tasks(...))`
 
         Args:
-            project_name (str):
+            project_name (str, optional):
                 Project Name
 
             batch_name (str, optional):
@@ -326,11 +395,28 @@ class ScaleClient:
                 The tags of a task; multiple tags can be
                 specified as a list.
 
+            include_attachment_url (bool):
+                If true, returns a pre-signed s3 url for the
+                attachment used to create the task.
+
+            limited_response (bool):
+                If true, returns task response of the following fields:
+                task_id, status, metadata, project, otherVersion.
+
+            limit (int):
+                Determines the task count per request (1-100)
+                For large sized tasks, use a smaller limit
+
         Yields:
             Generator[Task]:
                 Yields Task objects, can be iterated.
         """
 
+        if not project_name and not batch_name:
+            raise ValueError(
+                "At least one of project_name or batch_name must be provided."
+            )
+
         next_token = None
         has_more = True
 
@@ -348,15 +434,19 @@ class ScaleClient:
             created_after,
             created_before,
             tags,
+            include_attachment_url,
+            limited_response,
         )
 
+        if limit:
+            tasks_args["limit"] = limit
+
         while has_more:
             tasks_args["next_token"] = next_token
 
             tasks = self.tasks(**tasks_args)
             for task in tasks.docs:
                 yield task
-
             next_token = tasks.next_token
             has_more = tasks.has_more
 
@@ -375,6 +465,7 @@ class ScaleClient:
         created_after: str = None,
         created_before: str = None,
         tags: Union[List[str], str] = None,
+        include_attachment_url: bool = True,
     ) -> int:
         """Returns number of tasks with given filters.
 
@@ -429,6 +520,10 @@ class ScaleClient:
                 The tags of a task; multiple tags can be
                 specified as a list.
 
+            include_attachment_url (bool):
+                If true, returns a pre-signed s3 url for the
+                attachment used to create the task.
+
         Returns:
             int:
                 Returns number of tasks
@@ -448,6 +543,7 @@ class ScaleClient:
             created_after,
             created_before,
             tags,
+            include_attachment_url,
         )
 
         tasks_args["limit"] = 1
@@ -457,7 +553,7 @@ class ScaleClient:
 
     @staticmethod
     def _process_tasks_endpoint_args(
-        project_name: str,
+        project_name: str = None,
         batch_name: str = None,
         task_type: TaskType = None,
         status: TaskStatus = None,
@@ -470,8 +566,15 @@ class ScaleClient:
         created_after: str = None,
         created_before: str = None,
         tags: Union[List[str], str] = None,
+        include_attachment_url: bool = True,
+        limited_response: bool = None,
     ):
         """Generates args for /tasks endpoint."""
+        if not project_name and not batch_name:
+            raise ValueError(
+                "At least one of project_name or batch_name must be provided."
+            )
+
         tasks_args = {
             "start_time": created_after,
             "end_time": created_before,
@@ -483,10 +586,13 @@ class ScaleClient:
             "updated_before": updated_before,
             "updated_after": updated_after,
             "unique_id": unique_id,
+            "include_attachment_url": include_attachment_url,
         }
 
         if status:
             tasks_args["status"] = status.value
+        if limited_response:
+            tasks_args["limited_response"] = limited_response
         if task_type:
             tasks_args["type"] = task_type.value
         if review_status:
@@ -530,6 +636,7 @@ class ScaleClient:
         callback: str = "",
         calibration_batch: bool = False,
         self_label_batch: bool = False,
+        metadata: Dict = None,
     ) -> Batch:
         """Create a new Batch within a project.
         https://docs.scale.com/reference#batch-creation
@@ -550,6 +657,8 @@ class ScaleClient:
                 Only applicable for self serve projects.
                 Create a self_label batch by setting
                 the self_label_batch flag to true.
+            metadata (Dict):
+                Optional metadata to be stored at the TaskBatch level
 
         Returns:
             Batch: Created batch object
@@ -561,6 +670,7 @@ class ScaleClient:
             calibration_batch=calibration_batch,
             self_label_batch=self_label_batch,
             callback=callback,
+            metadata=metadata or {},
         )
         batchdata = self.api.post_request(endpoint, body=payload)
         return Batch(batchdata, self)
@@ -633,6 +743,9 @@ class ScaleClient:
                 The maximum value of `created_at` in UTC timezone
                 ISO format: 'YYYY-MM-DD HH:MM:SS.mmmmmm'
 
+            exclude_archived (bool):
+                A flag to exclude archived batches if True
+
             status (str):
                 Status to filter batches by
 
@@ -654,6 +767,7 @@ class ScaleClient:
         allowed_kwargs = {
             "start_time",
             "end_time",
+            "exclude_archived",
             "status",
             "project",
             "limit",
@@ -683,6 +797,7 @@ class ScaleClient:
         batch_status: BatchStatus = None,
         created_after: str = None,
         created_before: str = None,
+        exclude_archived: bool = False,
     ) -> Generator[Batch, None, None]:
         """`Generator` method to yield all batches with the given
         parameters.
@@ -705,6 +820,9 @@ class ScaleClient:
                 The maximum value of `created_at` in UTC timezone
                 ISO format: 'YYYY-MM-DD HH:MM:SS.mmmmmm'
 
+            exclude_archived (bool):
+                A flag to exclude archived batches if True
+
         Yields:
             Generator[Batch]:
                 Yields Batch, can be iterated.
@@ -719,6 +837,7 @@ class ScaleClient:
                 "end_time": created_before,
                 "project": project_name,
                 "offset": offset,
+                "exclude_archived": exclude_archived,
             }
 
             if batch_status:
@@ -730,6 +849,22 @@ class ScaleClient:
             offset += batches.limit
             has_more = batches.has_more
 
+    def set_batch_metadata(self, batch_name: str, metadata: Dict) -> Batch:
+        """Sets metadata for a TaskBatch.
+
+        Args:
+            batch_name (str):
+                Batch name
+            metadata (Dict):
+                Metadata to set for TaskBatch
+
+        Returns:
+            Batch
+        """
+        endpoint = f"batches/{Api.quote_string(batch_name)}/setMetadata"
+        batchdata = self.api.post_request(endpoint, body=metadata)
+        return Batch(batchdata, self)
+
     def create_project(
         self,
         project_name: str,
@@ -737,6 +872,7 @@ class ScaleClient:
         params: Dict = None,
         rapid: bool = False,
         studio: bool = False,
+        dataset_id: Optional[str] = None,
     ) -> Project:
         """Creates a new project.
         https://docs.scale.com/reference#project-creation
@@ -744,13 +880,21 @@ class ScaleClient:
         Args:
             project_name (str):
                 Project name
-
             task_type (TaskType):
                 Task Type i.e. `TaskType.ImageAnnotation`
-
             params (Dict):
-                Project parameters to be specificed.
+                Project parameters to be specified.
                 i.e. `{'instruction':'Please label the kittens'}`
+            rapid (bool):
+                Whether the project being created is a
+                Scale Rapid project
+            studio (bool):
+                Whether the project being created is a
+                 Scale Studio project
+            dataset_id (str):
+                Link this project to an existing Nucleus dataset.
+                All tasks annotated in this project will
+                be synced to the given dataset.
 
         Returns:
             Project: [description]
@@ -762,6 +906,7 @@ class ScaleClient:
             params=params,
             rapid=rapid,
             studio=studio,
+            datasetId=dataset_id,
         )
         projectdata = self.api.post_request(endpoint, body=payload)
         return Project(projectdata, self)
@@ -826,6 +971,23 @@ class ScaleClient:
         projectdata = self.api.post_request(endpoint, body=kwargs)
         return Project(projectdata, self)
 
+    def get_project_template(self, project_name: str) -> TaskTemplate:
+        """Gets the task template of a project if a template exists.
+        Throws an error if the project task-type does not support
+        Task Templates. Currently only TextCollection and Chat task
+        types support Task Templates.
+
+        Args:
+            project_name (str):
+                Project's name
+
+        Returns:
+            TaskTemplate
+        """
+        endpoint = f"projects/{Api.quote_string(project_name)}/taskTemplates"
+        template = self.api.get_request(endpoint)
+        return TaskTemplate(template, self)
+
     def upload_file(self, file: IO, **kwargs) -> File:
         """Upload file.
         Refer to Files API Reference:
@@ -949,7 +1111,11 @@ class ScaleClient:
         teammate_list = self.api.get_request(endpoint)
         return [Teammate(teammate, self) for teammate in teammate_list]
 
-    def invite_teammates(self, emails: List[str], role: TeammateRole) -> List[Teammate]:
+    def invite_teammates(
+        self,
+        emails: List[str],
+        role: TeammateRole,
+    ) -> List[Teammate]:
         """Invites a list of emails to your team.
 
         Args:
@@ -989,7 +1155,9 @@ class ScaleClient:
         teammate_list = self.api.post_request(endpoint, payload)
         return [Teammate(teammate, self) for teammate in teammate_list]
 
-    def list_studio_assignments(self) -> Dict[str, StudioLabelerAssignment]:
+    def list_studio_assignments(
+        self,
+    ) -> Dict[str, StudioLabelerAssignment]:
         """Returns a dictionary where the keys are user emails and the
         values are projects the user is assigned to.
 
@@ -999,8 +1167,12 @@ class ScaleClient:
         endpoint = "studio/assignments"
         raw_assignments = self.api.get_request(endpoint)
         assignments = {}
-        for (email, assigned_projects) in raw_assignments.items():
-            assignments[email] = StudioLabelerAssignment(assigned_projects, email, self)
+        for email, assigned_projects in raw_assignments.items():
+            assignments[email] = StudioLabelerAssignment(
+                assigned_projects,
+                email,
+                self,
+            )
         return assignments
 
     def add_studio_assignments(
@@ -1023,8 +1195,12 @@ class ScaleClient:
         }
         raw_assignments = self.api.post_request(endpoint, payload)
         assignments = {}
-        for (email, assigned_projects) in raw_assignments.items():
-            assignments[email] = StudioLabelerAssignment(assigned_projects, email, self)
+        for email, assigned_projects in raw_assignments.items():
+            assignments[email] = StudioLabelerAssignment(
+                assigned_projects,
+                email,
+                self,
+            )
         return assignments
 
     def remove_studio_assignments(
@@ -1047,8 +1223,12 @@ class ScaleClient:
         }
         raw_assignments = self.api.post_request(endpoint, payload)
         assignments = {}
-        for (email, assigned_projects) in raw_assignments.items():
-            assignments[email] = StudioLabelerAssignment(assigned_projects, email, self)
+        for email, assigned_projects in raw_assignments.items():
+            assignments[email] = StudioLabelerAssignment(
+                assigned_projects,
+                email,
+                self,
+            )
         return assignments
 
     def list_project_groups(self, project: str) -> List[StudioProjectGroup]:
@@ -1083,7 +1263,10 @@ class ScaleClient:
         """
         endpoint = f"studio/projects/{Api.quote_string(project)}/groups"
         payload = {"emails": emails, "name": project_group}
-        return StudioProjectGroup(self.api.post_request(endpoint, payload), self)
+        return StudioProjectGroup(
+            self.api.post_request(endpoint, payload),
+            self,
+        )
 
     def update_project_group(
         self,
@@ -1110,8 +1293,14 @@ class ScaleClient:
             f"studio/projects/{Api.quote_string(project)}"
             f"/groups/{Api.quote_string(project_group)}"
         )
-        payload = {"add_emails": add_emails, "remove_emails": remove_emails}
-        return StudioProjectGroup(self.api.put_request(endpoint, payload), self)
+        payload = {
+            "add_emails": add_emails,
+            "remove_emails": remove_emails,
+        }
+        return StudioProjectGroup(
+            self.api.put_request(endpoint, payload),
+            self,
+        )
 
     def list_studio_batches(self) -> List[StudioBatch]:
         """Returns a list with all pending studio batches,
@@ -1152,7 +1341,12 @@ class ScaleClient:
         Returns:
             List[StudioBatch]
         """
-        batches_names = list(map(lambda batch_name: {"name": batch_name}, batch_names))
+        batches_names = list(
+            map(
+                lambda batch_name: {"name": batch_name},
+                batch_names,
+            )
+        )
         endpoint = "studio/batches/set_priorities"
         payload = {"batches": batches_names}
         batches = self.api.post_request(endpoint, payload)
