@@ -19,7 +19,15 @@ HTTP_RETRY_ALLOWED_METHODS = frozenset({"GET", "POST", "DELETE"})
 class Api:
     """Internal Api reference for handling http operations"""
 
-    def __init__(self, api_key, user_agent_extension=None, api_instance_url=None):
+    def __init__(
+        self,
+        api_key,
+        user_agent_extension=None,
+        api_instance_url=None,
+        verify=None,
+        proxies=None,
+        cert=None,
+    ):
         if api_key == "" or api_key is None:
             raise ScaleException("Please provide a valid API Key.")
 
@@ -35,6 +43,10 @@ class Api:
         }
         self.base_api_url = api_instance_url or SCALE_API_BASE_URL_V1
 
+        self.verify = verify
+        self.proxies = proxies
+        self.cert = cert
+
     @staticmethod
     def _http_request(
         method,
@@ -45,8 +57,10 @@ class Api:
         body=None,
         files=None,
         data=None,
+        verify=None,
+        proxies=None,
+        cert=None,
     ) -> Response:
-
         https = requests.Session()
         retry_strategy = Retry(
             total=HTTP_TOTAL_RETRIES,
@@ -59,6 +73,11 @@ class Api:
         adapter = HTTPAdapter(max_retries=retry_strategy)
         https.mount("https://", adapter)
 
+        https.cert = cert if cert else None
+        https.verify = verify if verify else True
+        if proxies:
+            https.proxies.update(proxies)
+
         try:
             params = params or {}
             body = body or None
@@ -80,7 +99,6 @@ class Api:
 
     @staticmethod
     def _raise_on_respose(res: Response):
-
         try:
             message = res.json().get("error", res.text)
         except ValueError:
@@ -104,7 +122,19 @@ class Api:
 
         url = f"{self.base_api_url}/{endpoint}"
 
-        res = self._http_request(method, url, headers, auth, params, body, files, data)
+        res = self._http_request(
+            method,
+            url,
+            headers,
+            auth,
+            params,
+            body,
+            files,
+            data,
+            verify=self.verify,
+            proxies=self.proxies,
+            cert=self.cert,
+        )
 
         json = None
         if res.status_code == 200:
@@ -114,23 +144,32 @@ class Api:
                 # Some endpoints only return 'OK' message without JSON
                 return json
         elif res.status_code == 409 and "task" in endpoint and body.get("unique_id"):
-            retry_history = res.raw.retries.history
+            try:
+                retry_history = res.raw.retries.history
+            except AttributeError:
+                retry_history = []
             # Example RequestHistory tuple
             # RequestHistory(method='POST',
             #   url='/v1/task/imageannotation',
             #   error=None,
             #   status=409,
             #   redirect_location=None)
-            if retry_history != ():
-                # See if the first retry was a 500 error
-                if retry_history[0][3] == 500:
-                    uuid = body["unique_id"]
-                    newUrl = f"{self.base_api_url}/tasks?unique_id={uuid}"
-                    # grab task from api
-                    newRes = self._http_request(
-                        "GET", newUrl, headers=headers, auth=auth
-                    )
-                    json = newRes.json()["docs"][0]
+
+            if retry_history and retry_history[0][3] >= 500:
+                uuid = body["unique_id"]
+                new_url = f"{self.base_api_url}/tasks?unique_id={uuid}"
+
+                # Perform a GET request to retrieve task data
+                new_res = self._http_request("GET", new_url, headers=headers, auth=auth)
+
+                if new_res.status_code == 200:
+                    new_res_data = new_res.json()
+                    if new_res_data["docs"]:
+                        json = new_res_data["docs"][0]
+                    else:
+                        self._raise_on_respose(new_res)
+                else:
+                    self._raise_on_respose(new_res)
             else:
                 self._raise_on_respose(res)
         else:
@@ -148,19 +187,24 @@ class Api:
         return self._api_request(
             "POST",
             endpoint,
-            headers=self._headers
-            if files is None
-            else self._headers_multipart_form_data,
+            headers=(
+                self._headers if files is None else self._headers_multipart_form_data
+            ),
             auth=self._auth,
             body=body,
             files=files,
             data=data,
         )
 
-    def delete_request(self, endpoint, params=None):
+    def delete_request(self, endpoint, params=None, body=None):
         """Generic DELETE Request Wrapper"""
         return self._api_request(
-            "DELETE", endpoint, headers=self._headers, auth=self._auth, params=params
+            "DELETE",
+            endpoint,
+            headers=self._headers,
+            auth=self._auth,
+            params=params,
+            body=body,
         )
 
     def put_request(self, endpoint, body=None, params=None):
